var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sleep = exports.register = void 0;
const embedded = require("@volar/language-service");
const vscode = require("vscode-languageserver");
const protocol_1 = require("../../protocol");
const types_1 = require("../../types");
function register(connection, projects, initParams, initOptions, cancelHost, semanticTokensLegend, runtime, documents) {
    let lastCompleteUri;
    let lastCompleteLs;
    let lastCodeLensLs;
    let lastCodeActionLs;
    let lastCallHierarchyLs;
    connection.onDocumentFormatting((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.format(params.textDocument.uri, params.options, undefined, undefined, token);
        });
    }));
    connection.onDocumentRangeFormatting((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.format(params.textDocument.uri, params.options, params.range, undefined, token);
        });
    }));
    connection.onDocumentOnTypeFormatting((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.format(params.textDocument.uri, params.options, undefined, params, token);
        });
    }));
    connection.onSelectionRanges((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.getSelectionRanges(params.textDocument.uri, params.positions, token);
        });
    }));
    connection.onFoldingRanges((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.getFoldingRanges(params.textDocument.uri, token);
        });
    }));
    connection.languages.onLinkedEditingRange((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.findLinkedEditingRanges(params.textDocument.uri, params.position, token);
        });
    }));
    connection.onDocumentSymbol((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.findDocumentSymbols(params.textDocument.uri, token);
        });
    }));
    connection.onDocumentColor((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.findDocumentColors(params.textDocument.uri, token);
        });
    }));
    connection.onColorPresentation((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.getColorPresentations(params.textDocument.uri, params.color, params.range, token);
        });
    }));
    connection.onCompletion((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, (service) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            lastCompleteUri = params.textDocument.uri;
            lastCompleteLs = service;
            const document = (_a = documents.data.uriGet(params.textDocument.uri)) === null || _a === void 0 ? void 0 : _a.getDocument();
            const list = yield service.doComplete(params.textDocument.uri, params.position, params.context, token);
            for (const item of list.items) {
                fixTextEdit(item);
            }
            if (!initOptions.fullCompletionList && document) {
                list.items = list.items.filter(item => {
                    var _a, _b;
                    const range = item.textEdit ? (vscode.InsertReplaceEdit.is(item.textEdit)
                        ? item.textEdit.replace
                        : item.textEdit.range) : ((_a = list.itemDefaults) === null || _a === void 0 ? void 0 : _a.editRange) ? (vscode.Range.is(list.itemDefaults.editRange)
                        ? list.itemDefaults.editRange
                        : list.itemDefaults.editRange.replace) : undefined;
                    if (range) {
                        const sourceText = document.getText(range).toLowerCase();
                        if (sourceText.trim()) {
                            let filterText = ((_b = item.filterText) !== null && _b !== void 0 ? _b : item.label).toLowerCase();
                            for (const char of sourceText) {
                                const index = filterText.indexOf(char);
                                if (index === -1) {
                                    return false;
                                }
                                filterText = filterText.slice(index + 1);
                            }
                        }
                    }
                    return true;
                });
            }
            return list;
        }));
    }));
    connection.onCompletionResolve((item, token) => __awaiter(this, void 0, void 0, function* () {
        if (lastCompleteUri && lastCompleteLs) {
            item = yield lastCompleteLs.doCompletionResolve(item, token);
            fixTextEdit(item);
        }
        return item;
    }));
    connection.onHover((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.doHover(params.textDocument.uri, params.position, token);
        });
    }));
    connection.onSignatureHelp((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.getSignatureHelp(params.textDocument.uri, params.position, params.context, token);
        });
    }));
    connection.onPrepareRename((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.prepareRename(params.textDocument.uri, params.position, token);
        });
    }));
    connection.onRenameRequest((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.doRename(params.textDocument.uri, params.position, params.newName, token);
        });
    }));
    connection.onCodeLens((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, (service) => __awaiter(this, void 0, void 0, function* () {
            lastCodeLensLs = service;
            return service.doCodeLens(params.textDocument.uri, token);
        }));
    }));
    connection.onCodeLensResolve((codeLens, token) => __awaiter(this, void 0, void 0, function* () {
        var _b;
        return (_b = yield (lastCodeLensLs === null || lastCodeLensLs === void 0 ? void 0 : lastCodeLensLs.doCodeLensResolve(codeLens, token))) !== null && _b !== void 0 ? _b : codeLens;
    }));
    connection.onExecuteCommand((params) => __awaiter(this, void 0, void 0, function* () {
        if (params.command === embedded.showReferencesCommand
            && params.arguments) {
            const args = params.arguments;
            yield connection.sendNotification(protocol_1.ShowReferencesNotification.type, {
                textDocument: {
                    uri: args[0],
                },
                position: args[1],
                references: args[2],
            });
        }
    }));
    connection.onCodeAction((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, (service) => __awaiter(this, void 0, void 0, function* () {
            var _c, _d, _e;
            lastCodeActionLs = service;
            let codeActions = (_c = yield service.doCodeActions(params.textDocument.uri, params.range, params.context, token)) !== null && _c !== void 0 ? _c : [];
            for (const codeAction of codeActions) {
                if (codeAction.data && typeof codeAction.data === 'object') {
                    codeAction.data.uri = params.textDocument.uri;
                }
                else {
                    codeAction.data = { uri: params.textDocument.uri };
                }
            }
            if (!((_e = (_d = initParams.capabilities.textDocument) === null || _d === void 0 ? void 0 : _d.codeAction) === null || _e === void 0 ? void 0 : _e.disabledSupport)) {
                codeActions = codeActions.filter(codeAction => !codeAction.disabled);
            }
            return codeActions;
        }));
    }));
    connection.onCodeActionResolve((codeAction, token) => __awaiter(this, void 0, void 0, function* () {
        var _f;
        return (_f = yield lastCodeActionLs.doCodeActionResolve(codeAction, token)) !== null && _f !== void 0 ? _f : codeAction;
    }));
    connection.onReferences((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.findReferences(params.textDocument.uri, params.position, token);
        });
    }));
    connection.onRequest(protocol_1.FindFileReferenceRequest.type, (params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.findFileReferences(params.textDocument.uri, token);
        });
    }));
    connection.onImplementation((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.findImplementations(params.textDocument.uri, params.position, token);
        });
    }));
    connection.onDefinition((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.findDefinition(params.textDocument.uri, params.position, token);
        });
    }));
    connection.onTypeDefinition((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.findTypeDefinition(params.textDocument.uri, params.position, token);
        });
    }));
    connection.onDocumentHighlight((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.findDocumentHighlights(params.textDocument.uri, params.position, token);
        });
    }));
    connection.onDocumentLinks((params, token) => __awaiter(this, void 0, void 0, function* () {
        return yield worker(params.textDocument.uri, token, service => {
            return service.findDocumentLinks(params.textDocument.uri, token);
        });
    }));
    connection.onWorkspaceSymbol((params, token) => __awaiter(this, void 0, void 0, function* () {
        let results = [];
        for (const _workspace of projects.workspaces.values()) {
            const workspace = yield _workspace;
            let projects = [...workspace.projects.values()];
            projects = projects.length ? projects : [workspace.getInferredProject()];
            for (const project of projects) {
                if (token.isCancellationRequested)
                    return;
                const service = (yield project).getLanguageService();
                results = results.concat(yield service.findWorkspaceSymbols(params.query, token));
            }
        }
        return results;
    }));
    connection.languages.callHierarchy.onPrepare((params, token) => __awaiter(this, void 0, void 0, function* () {
        var _g;
        return (_g = yield worker(params.textDocument.uri, token, (service) => __awaiter(this, void 0, void 0, function* () {
            lastCallHierarchyLs = service;
            return service.callHierarchy.doPrepare(params.textDocument.uri, params.position, token);
        }))) !== null && _g !== void 0 ? _g : [];
    }));
    connection.languages.callHierarchy.onIncomingCalls((params, token) => __awaiter(this, void 0, void 0, function* () {
        var _h;
        return (_h = yield (lastCallHierarchyLs === null || lastCallHierarchyLs === void 0 ? void 0 : lastCallHierarchyLs.callHierarchy.getIncomingCalls(params.item, token))) !== null && _h !== void 0 ? _h : [];
    }));
    connection.languages.callHierarchy.onOutgoingCalls((params, token) => __awaiter(this, void 0, void 0, function* () {
        var _j;
        return (_j = yield (lastCallHierarchyLs === null || lastCallHierarchyLs === void 0 ? void 0 : lastCallHierarchyLs.callHierarchy.getOutgoingCalls(params.item, token))) !== null && _j !== void 0 ? _j : [];
    }));
    connection.languages.semanticTokens.on((params, token, _, resultProgress) => __awaiter(this, void 0, void 0, function* () {
        var _k;
        yield sleep(200);
        return (_k = yield worker(params.textDocument.uri, token, (service) => __awaiter(this, void 0, void 0, function* () {
            return yield (service === null || service === void 0 ? void 0 : service.getSemanticTokens(params.textDocument.uri, undefined, semanticTokensLegend, token, tokens => resultProgress === null || resultProgress === void 0 ? void 0 : resultProgress.report(tokens)));
        }))) !== null && _k !== void 0 ? _k : { data: [] };
    }));
    connection.languages.semanticTokens.onRange((params, token, _, resultProgress) => __awaiter(this, void 0, void 0, function* () {
        var _l;
        yield sleep(200);
        return (_l = yield worker(params.textDocument.uri, token, (service) => __awaiter(this, void 0, void 0, function* () {
            return yield (service === null || service === void 0 ? void 0 : service.getSemanticTokens(params.textDocument.uri, params.range, semanticTokensLegend, token, tokens => resultProgress === null || resultProgress === void 0 ? void 0 : resultProgress.report(tokens)));
        }))) !== null && _l !== void 0 ? _l : { data: [] };
    }));
    connection.languages.diagnostics.on((params, token, _workDoneProgressReporter, resultProgressReporter) => __awaiter(this, void 0, void 0, function* () {
        const result = yield worker(params.textDocument.uri, token, service => {
            const tsToken = cancelHost.createCancellationToken(token);
            const mode = initOptions.serverMode === types_1.ServerMode.PartialSemantic ? 'semantic'
                : initOptions.serverMode === types_1.ServerMode.Syntactic ? 'syntactic'
                    : 'all';
            return service.doValidation(params.textDocument.uri, tsToken, mode, errors => {
                // resultProgressReporter is undefined in vscode
                resultProgressReporter === null || resultProgressReporter === void 0 ? void 0 : resultProgressReporter.report({
                    relatedDocuments: {
                        [params.textDocument.uri]: {
                            kind: vscode.DocumentDiagnosticReportKind.Full,
                            items: errors,
                        },
                    },
                });
            });
        });
        return {
            kind: vscode.DocumentDiagnosticReportKind.Full,
            items: result !== null && result !== void 0 ? result : [],
        };
    }));
    connection.languages.inlayHint.on((params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, (service) => __awaiter(this, void 0, void 0, function* () {
            return service.getInlayHints(params.textDocument.uri, params.range, token);
        }));
    }));
    // TODO: connection.languages.inlayHint.resolve
    connection.workspace.onWillRenameFiles((params, token) => __awaiter(this, void 0, void 0, function* () {
        const config = yield connection.workspace.getConfiguration('volar.updateImportsOnFileMove.enabled');
        if (!config) {
            return null;
        }
        const _edits = yield Promise.all(params.files.map((file) => __awaiter(this, void 0, void 0, function* () {
            var _m;
            return (_m = yield worker(file.oldUri, token, service => {
                var _a;
                return (_a = service.getEditsForFileRename(file.oldUri, file.newUri, token)) !== null && _a !== void 0 ? _a : null;
            })) !== null && _m !== void 0 ? _m : null;
        })));
        const edits = _edits.filter((edit) => !!edit);
        if (edits.length) {
            embedded.mergeWorkspaceEdits(edits[0], ...edits.slice(1));
            return edits[0];
        }
        return null;
    }));
    connection.onRequest(protocol_1.AutoInsertRequest.type, (params, token) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, token, service => {
            return service.doAutoInsert(params.textDocument.uri, params.position, params.options, token);
        });
    }));
    function worker(uri, token, cb) {
        return new Promise(resolve => {
            runtime.timer.setImmediate(() => __awaiter(this, void 0, void 0, function* () {
                if (token.isCancellationRequested) {
                    resolve(undefined);
                    return;
                }
                const service = yield getLanguageService(uri);
                if (service) {
                    try { // handle TS cancel throw
                        const result = yield cb(service);
                        if (token.isCancellationRequested) {
                            resolve(undefined);
                            return;
                        }
                        resolve(result);
                    }
                    catch (_a) {
                        resolve(undefined);
                        return;
                    }
                }
                else {
                    resolve(undefined);
                }
            }));
        });
    }
    function getLanguageService(uri) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const project = (_a = (yield projects.getProject(uri))) === null || _a === void 0 ? void 0 : _a.project;
            return project === null || project === void 0 ? void 0 : project.getLanguageService();
        });
    }
    function fixTextEdit(item) {
        var _a, _b, _c, _d;
        const insertReplaceSupport = (_d = (_c = (_b = (_a = initParams.capabilities.textDocument) === null || _a === void 0 ? void 0 : _a.completion) === null || _b === void 0 ? void 0 : _b.completionItem) === null || _c === void 0 ? void 0 : _c.insertReplaceSupport) !== null && _d !== void 0 ? _d : false;
        if (!insertReplaceSupport) {
            if (item.textEdit && vscode.InsertReplaceEdit.is(item.textEdit)) {
                item.textEdit = vscode.TextEdit.replace(item.textEdit.insert, item.textEdit.newText);
            }
        }
    }
}
exports.register = register;
function sleep(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}
exports.sleep = sleep;
//# sourceMappingURL=languageFeatures.js.map