var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startCommonLanguageServer = void 0;
const language_service_1 = require("@volar/language-service");
const l10n = require("@vscode/l10n");
const vscode = require("vscode-languageserver");
const vscode_uri_1 = require("vscode-uri");
const types_1 = require("../types");
const cancellationPipe_1 = require("./cancellationPipe");
const configurationHost_1 = require("./configurationHost");
const documents_1 = require("./documents");
const registerFeatures_1 = require("./utils/registerFeatures");
const serverConfig_1 = require("./utils/serverConfig");
const workspaces_1 = require("./workspaces");
function startCommonLanguageServer(connection, getCtx) {
    let initParams;
    let options;
    let roots = [];
    let fsHost;
    let projects;
    let configurationHost;
    let plugins;
    let documents;
    let context;
    connection.onInitialize((_params) => __awaiter(this, void 0, void 0, function* () {
        var _a, _b, _c, _d;
        initParams = _params;
        options = initParams.initializationOptions;
        context = getCtx(options);
        plugins = context.plugins.map(plugin => plugin(options));
        documents = (0, documents_1.createDocuments)(context.runtimeEnv, connection);
        if (options.l10n) {
            yield l10n.config({ uri: options.l10n.location });
        }
        if (((_a = initParams.capabilities.workspace) === null || _a === void 0 ? void 0 : _a.workspaceFolders) && initParams.workspaceFolders) {
            roots = initParams.workspaceFolders.map(folder => vscode_uri_1.URI.parse(folder.uri));
        }
        else if (initParams.rootUri) {
            roots = [vscode_uri_1.URI.parse(initParams.rootUri)];
        }
        else if (initParams.rootPath) {
            roots = [vscode_uri_1.URI.file(initParams.rootPath)];
        }
        const result = {
            capabilities: {
                textDocumentSync: (_b = options.textDocumentSync) !== null && _b !== void 0 ? _b : vscode.TextDocumentSyncKind.Incremental,
            },
        };
        configurationHost = ((_c = initParams.capabilities.workspace) === null || _c === void 0 ? void 0 : _c.configuration) ? (0, configurationHost_1.createConfigurationHost)(initParams, connection) : undefined;
        let lsPlugins = {};
        const ts = options.typescript ? context.runtimeEnv.loadTypescript(options.typescript.tsdk) : undefined;
        for (const root of roots) {
            if (root.scheme === 'file') {
                let config = (_d = (0, serverConfig_1.loadConfig)(root.path, options.configFilePath)) !== null && _d !== void 0 ? _d : {};
                for (const plugin of plugins) {
                    if (plugin.resolveConfig) {
                        config = plugin.resolveConfig(config, { typescript: ts });
                    }
                }
                if (config.plugins) {
                    lsPlugins = Object.assign(Object.assign({}, lsPlugins), config.plugins);
                }
            }
        }
        (0, registerFeatures_1.setupCapabilities)(result.capabilities, options, plugins, getSemanticTokensLegend(), lsPlugins);
        yield createLanguageServiceHost();
        try {
            // show version on LSP logs
            const packageJson = require('../package.json');
            result.serverInfo = {
                name: packageJson.name,
                version: packageJson.version,
            };
        }
        catch (_e) { }
        return result;
    }));
    connection.onInitialized(() => {
        var _a, _b, _c;
        fsHost === null || fsHost === void 0 ? void 0 : fsHost.ready(connection);
        configurationHost === null || configurationHost === void 0 ? void 0 : configurationHost.ready();
        if ((_a = initParams.capabilities.workspace) === null || _a === void 0 ? void 0 : _a.workspaceFolders) {
            connection.workspace.onDidChangeWorkspaceFolders(e => {
                for (const folder of e.added) {
                    projects === null || projects === void 0 ? void 0 : projects.add(vscode_uri_1.URI.parse(folder.uri));
                }
                for (const folder of e.removed) {
                    projects === null || projects === void 0 ? void 0 : projects.remove(vscode_uri_1.URI.parse(folder.uri));
                }
            });
        }
        if (options.serverMode !== types_1.ServerMode.Syntactic
            && !options.disableFileWatcher
            && ((_c = (_b = initParams.capabilities.workspace) === null || _b === void 0 ? void 0 : _b.didChangeWatchedFiles) === null || _c === void 0 ? void 0 : _c.dynamicRegistration)) {
            const exts = plugins.map(plugin => plugin.watchFileExtensions).flat();
            if (exts.length) {
                connection.client.register(vscode.DidChangeWatchedFilesNotification.type, {
                    watchers: [
                        {
                            globPattern: `**/*.{${exts.join(',')}}`
                        },
                    ]
                });
            }
        }
    });
    connection.onShutdown(() => __awaiter(this, void 0, void 0, function* () {
        if (projects) {
            for (const workspace of projects.workspaces) {
                (yield workspace[1]).dispose();
            }
        }
    }));
    connection.listen();
    function createLanguageServiceHost() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const ts = options.typescript ? context.runtimeEnv.loadTypescript(options.typescript.tsdk) : undefined;
            fsHost = ts ? context.runtimeEnv.createFileSystemHost(ts, initParams.capabilities, context.runtimeEnv, options) : undefined;
            const tsLocalized = options.typescript && initParams.locale ? yield context.runtimeEnv.loadTypescriptLocalized(options.typescript.tsdk, initParams.locale) : undefined;
            const cancelTokenHost = (0, cancellationPipe_1.createCancellationTokenHost)(options.cancellationPipeName);
            const _projects = (0, workspaces_1.createWorkspaces)({
                server: context,
                fileSystemHost: fsHost,
                configurationHost,
                ts,
                tsLocalized,
                initParams: initParams,
                initOptions: options,
                documents,
                cancelTokenHost,
                plugins,
            });
            projects = _projects;
            for (const root of roots) {
                projects.add(root);
            }
            (yield Promise.resolve().then(() => require('./features/customFeatures'))).register(connection, projects, context.runtimeEnv);
            (yield Promise.resolve().then(() => require('./features/languageFeatures'))).register(connection, projects, initParams, options, cancelTokenHost, getSemanticTokensLegend(), context.runtimeEnv, documents);
            for (const plugin of plugins) {
                (_a = plugin.onInitialized) === null || _a === void 0 ? void 0 : _a.call(plugin, getLanguageService, context.runtimeEnv);
            }
            function getLanguageService(uri) {
                var _a;
                return __awaiter(this, void 0, void 0, function* () {
                    const project = (_a = (yield projects.getProject(uri))) === null || _a === void 0 ? void 0 : _a.project;
                    return project === null || project === void 0 ? void 0 : project.getLanguageService();
                });
            }
        });
    }
    function getSemanticTokensLegend() {
        if (!options.semanticTokensLegend) {
            return language_service_1.standardSemanticTokensLegend;
        }
        return {
            tokenTypes: [...language_service_1.standardSemanticTokensLegend.tokenTypes, ...options.semanticTokensLegend.tokenTypes],
            tokenModifiers: [...language_service_1.standardSemanticTokensLegend.tokenModifiers, ...options.semanticTokensLegend.tokenModifiers],
        };
    }
}
exports.startCommonLanguageServer = startCommonLanguageServer;
//# sourceMappingURL=server.js.map